@@ -27,7 +27,7 @@ struct keymap {
 };
 typedef struct keymap keymap;
 
-int PinyinToZuin( ChewingData *pgdata, char *pinyinKeySeq, char *zuinKeySeq, char *zuinKeySeqAlt);
+int PinyinToZuin( ChewingData *pgdata, const char *pinyinKeySeq, char *zuinKeySeq, char *zuinKeySeqAlt);
 int InitPinyin( ChewingData *pgdata, const char * );
 void TerminatePinyin( ChewingData *pgdata );
 
