@@ -14,13 +14,12 @@
 #define IS_USER_PHRASE 1
 #define IS_DICT_PHRASE 0
 
-void InitTree( const char * );
-int Phrasing( PhrasingOutput *ppo, uint16 phoneSeq[], int nPhoneSeq, 
-		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
-		IntervalType selectInterval[], int nSelect, 
-		int bArrBrkpt[], int bUserArrCnnct[] );
+int InitTree( ChewingData *pgdata, const char *prefix );
+void TerminateTree( ChewingData *pgdata );
+
+int Phrasing( ChewingData *pgdata );
 int IsIntersect( IntervalType in1, IntervalType in2 );
 
-int TreeFindPhrase( int begin, int end, const uint16 *phoneSeq );
+int TreeFindPhrase( ChewingData *pgdata, int begin, int end, const uint16_t *phoneSeq );
 
 #endif
