@@ -13,7 +13,7 @@
 #include "chewing-utf8-util.h"
 
 /* Table of UTF-8 length */
-static char utf8len_tab[256] =
+static const char utf8len_tab[256] =
 {
 	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
 	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
@@ -48,7 +48,7 @@ int ueBytesFromChar( unsigned char b )
 int ueStrNBytes( const char *str, int n )
 {
 	int i = 0, len = 0;
-	char *iter = (char *) str;
+	const char *iter = str;
 	for ( i = 0; i < n; i++ ) {
 		len += ueBytesFromChar( iter[ len ] );
 	}
@@ -66,6 +66,16 @@ int ueStrNCpy( char dest[], const char *src, size_t n, int end )
 	return len;
 }
 
+const char *ueConstStrSeek( const char *src, size_t n )
+{
+	size_t i = 0;
+	const char *iter = src;
+	for ( i = 0; i < n; i++ ) {
+		iter += ueBytesFromChar( iter[0] );
+	}
+	return iter;
+}
+
 char *ueStrSeek( char *src, size_t n )
 {
 	size_t i = 0;
