@@ -20,7 +20,6 @@
 #include "chewing-private.h"
 #include "zuin-private.h"
 #include "chewingio.h"
-#include "private.h"
 
 /**
  * @param ctx handle to Chewing IM context
@@ -42,7 +41,7 @@ CHEWING_API char *chewing_commit_String( ChewingContext *ctx )
 	int i;
 	char *s = (char *) calloc(
 		1 + ctx->output->nCommitStr,
-		sizeof(char) * MAX_UTF8_SIZE );
+		MAX_UTF8_SIZE );
 	if ( s ) {
 		for ( i = 0; i < ctx->output->nCommitStr; i++ ) {
 			strcat( s, (char *) (ctx->output->commitStr[ i ].s) );
@@ -66,7 +65,7 @@ CHEWING_API char *chewing_buffer_String( ChewingContext *ctx )
 	int i;
 	char *s = (char *) calloc(
 		1 + ctx->output->chiSymbolBufLen,
-		sizeof(char) * MAX_UTF8_SIZE );
+		MAX_UTF8_SIZE );
 	if ( s ) {
 		for ( i = 0; i < ctx->output->chiSymbolBufLen; i++ ) {
 			strcat( s, (char *) (ctx->output->chiSymbolBuf[ i ].s) );
@@ -148,7 +147,7 @@ CHEWING_API void chewing_cand_Enumerate( ChewingContext *ctx )
 
 CHEWING_API int chewing_cand_hasNext( ChewingContext *ctx )
 {
-	return (ctx->cand_no < ctx->output->pci->nTotalChoice);	
+	return (ctx->cand_no < ctx->output->pci->nTotalChoice);
 }
 
 CHEWING_API char *chewing_cand_String( ChewingContext *ctx )
@@ -199,7 +198,7 @@ CHEWING_API char *chewing_aux_String( ChewingContext *ctx )
 	int i;
 	char *msg = (char *) calloc(
 		1 + ctx->output->showMsgLen,
-		sizeof(char) * MAX_UTF8_SIZE );
+		MAX_UTF8_SIZE );
 	if ( msg ) {
 		for ( i = 0; i < ctx->output->showMsgLen; ++i )
 			strcat( msg, (char *)(ctx->output->showMsg[ i ].s) );
@@ -209,12 +208,12 @@ CHEWING_API char *chewing_aux_String( ChewingContext *ctx )
 }
 
 CHEWING_API int chewing_keystroke_CheckIgnore( ChewingContext *ctx )
-{ 
+{
 	return !!(ctx->output->keystrokeRtn & KEYSTROKE_IGNORE);
-} 
+}
 
 CHEWING_API int chewing_keystroke_CheckAbsorb( ChewingContext *ctx )
-{ 
+{
 	return !!(ctx->output->keystrokeRtn & KEYSTROKE_ABSORB);
 }
 
@@ -233,7 +232,7 @@ CHEWING_API int chewing_kbtype_hasNext( ChewingContext *ctx )
 	return ctx->kb_no < KB_TYPE_NUM;
 }
 
-extern char *kb_type_str[];
+extern const char * const kb_type_str[];
 
 CHEWING_API char *chewing_kbtype_String( ChewingContext *ctx )
 {
